#include "stdlib.h"

#include "MonochromeLib.h"

#include "objetcs.h"
#include "time.h"

#define JUMP_DY -5
#define JUMP_AY 0.38

#define POS_X(x) (x*8 + 80)
#define POS_Y(y) (24+(y*10))

Hero * hero_create(unsigned char ** bmp) {
	Hero * hero = (Hero *) malloc(sizeof(Hero));	
	hero->x = 8;	
	hero->y = 61-30;
	hero->dx = 0;	
	hero->dy = 0;
	hero->ax = 0;	
	hero->ay = 0;
	
	hero->y_stop = -1;
	
	hero->bmp = bmp;
	hero->bmp_current = 0;
	
	hero->is_jmp = 0;
	hero->is_fall = 0;
	
	hero->tick_last = 0;
	hero->TICK_NEXT = 2;
	
	return hero;
}
void hero_reset(Hero * hero) {
	hero->x = 8;	
	hero->y = 61-30;
	hero->dx = 0;	
	hero->dy = 0;
	hero->ax = 0;	
	hero->ay = 0;
	
	hero->y_stop = -1;
	
	hero->bmp_current = 0;
	
	hero->is_jmp = 0;
	hero->is_fall = 0;
	
	hero->tick_last = 0;
	hero->TICK_NEXT = 2;
}

void hero_jump_start(Hero * hero) {
	if(!hero->is_jmp) {
		hero->dy = JUMP_DY;
		hero->ay = JUMP_AY;		
		hero->is_jmp = 1;
		hero->bmp_current = 1;
		hero->y_stop = hero->y;
	}
}
void hero_jump_reset(Hero * hero) {
	if(hero->is_jmp) {
		hero->ay = 0;
		hero->dy = 0;
		hero->is_jmp = 0;
		if(!hero->is_fall)
			hero->bmp_current = 0;
	}
}
void hero_falling_start(Hero * hero) {
	hero->is_fall = 1;
	hero->bmp_current = 2;
}

void hero_update(Hero * hero) {
	// Update mouvement
	if(time_getTicks() - hero->tick_last > hero->TICK_NEXT) {
		hero->dx += hero->ax;
		hero->dy += hero->ay ;
		
		hero->x += hero->dx;
		hero->y += hero->dy;
		
		if(hero->is_jmp && hero->y > hero->y_stop) {
			hero->y = hero->y_stop;
			
			hero_jump_reset(hero);
		}
		
		hero->tick_last = time_getTicks();
	}
}

Boat * boat_create(unsigned char ** bmp) {
	Boat * boat = (Boat *) malloc(sizeof(Boat));
	
	boat->x = 100;
	boat->y = 31;
	
	boat->position = 1;
	
	boat->bmp = bmp;
	
	boat->tick_last = 0;
	
	boat->TICK_NEXT = 30;
	
	return boat;
}
void boat_reset(Boat * boat) {
	boat->x = 100;
	boat->y = 31;
	
	boat->position = 1;
	
	boat->tick_last = 0;
	
	boat->TICK_NEXT = 30;
}
void boat_draw(Boat * boat) {
	ML_bmp_16_or_cl(boat->bmp[0], boat->x, boat->y);
	ML_bmp_16_or_cl(boat->bmp[1], boat->x+16, boat->y);
}
void boat_update(Boat * boat) {
	// Update position
	if(time_getTicks() - boat->tick_last > boat->TICK_NEXT) {
		if(boat->position == 1) {
			boat->y = 30;
			boat->position = 2;
			boat->TICK_NEXT = 4*30;
		} else if(boat->position == 2) {
			boat->y = 31;
			boat->position = 1;
			boat->TICK_NEXT = 1*30;
		}
		boat->tick_last = time_getTicks();
	}
}

Water * water_create(Hero * hero, char type) {
	Water * water = (Water *) malloc(sizeof(Water));
	
	water->type = type;
	
	water_reset(hero, water);
	
	water->TICK_NEXT = 2;
	
	return water;
}
void water_reset(Hero * hero, Water * water) {
	water->x = hero->x + 1;
	water->y = hero->y + 15;
	
	water->dx = 0;	
	water->dy = 0;
	water->ax = 0;	
	water->ay = 0;
	
	water->dx = -0.8;
	
	switch(water->type) {
		default:
		case 1:			
			water->dy = -2;
			water->ay = 0.35;
		break;
		case 2:
			water->dy = -2;
			water->ay = 0.45;		
		break;
		case 3:
			water->dy = -2;
			water->ay = 0.40;		
		break;
		case 4:			
			water->dy = -2;
			water->ay = 0.48;
		break;
		case 5:
			water->dy = -2;
			water->ay = 0.43;		
		break;
		case 6:
			water->dy = -2;
			water->ay = 0.38;		
		break;
	}
	
	water->y_stop = water->y;
	
	water->tick_last = 0;
}
void water_draw(Water * water) {
	ML_pixel(water->x, water->y, ML_BLACK);
}
void water_update(Water * water) {
	// Update mouvement
	if(time_getTicks() - water->tick_last > water->TICK_NEXT) {
		water->dx += water->ax;
		water->dy += water->ay ;
		
		water->x += water->dx;
		water->y += water->dy;
		
		water->tick_last = time_getTicks();
	}
}

Stone * stone_create(int x, int y) {
	Stone * stone = (Stone *) malloc(sizeof(Stone));
	
	stone->x = x;
	stone->y = y;
	
	return stone;
}
void map_load(Stone * stones[60], char type) {
	int i = 0, j = 0;
	int rand1, rand2;
	int x, x_tmp, x_max, y_tmp;
	
	switch(type) {
		default:
		case 1:
			stones[i++] = stone_create( POS_X(9), POS_Y(0) );
			stones[i++] = stone_create( POS_X(10), POS_Y(1) );
			stones[i++] = stone_create( POS_X(10), POS_Y(2) );
			stones[i++] = stone_create( POS_X(9), POS_Y(3) );
			
			stones[i++] = stone_create( POS_X(12), POS_Y(0) );
			stones[i++] = stone_create( POS_X(11), POS_Y(1) );
			stones[i++] = stone_create( POS_X(12), POS_Y(2) );
			stones[i++] = stone_create( POS_X(11), POS_Y(3) );
			
			stones[i++] = stone_create( POS_X(16), POS_Y(0) );
			stones[i++] = stone_create( POS_X(17), POS_Y(1) );
			stones[i++] = stone_create( POS_X(16), POS_Y(2) );
			stones[i++] = stone_create( POS_X(17), POS_Y(3) );
			
			stones[i++] = stone_create( POS_X(18), POS_Y(0) );
			stones[i++] = stone_create( POS_X(18), POS_Y(1) );
			stones[i++] = stone_create( POS_X(19), POS_Y(2) );
			stones[i++] = stone_create( POS_X(19), POS_Y(3) );
			
			stones[i++] = stone_create( POS_X(23), POS_Y(0) );
			stones[i++] = stone_create( POS_X(24), POS_Y(1) );
			stones[i++] = stone_create( POS_X(26), POS_Y(2) );
			stones[i++] = stone_create( POS_X(26), POS_Y(3) ); // 20		
			
			
			stones[i++] = stone_create( POS_X(34), POS_Y(0) );
			stones[i++] = stone_create( POS_X(34), POS_Y(1) );
			stones[i++] = stone_create( POS_X(34), POS_Y(2) );		
			stones[i++] = stone_create( POS_X(37), POS_Y(0) );
			stones[i++] = stone_create( POS_X(38), POS_Y(1) );
			stones[i++] = stone_create( POS_X(37), POS_Y(2) );
			
			stones[i++] = stone_create( POS_X(42), POS_Y(0) );
			stones[i++] = stone_create( POS_X(42), POS_Y(2) );
			stones[i++] = stone_create( POS_X(42), POS_Y(3) );
			stones[i++] = stone_create( POS_X(46), POS_Y(0) );
			stones[i++] = stone_create( POS_X(45), POS_Y(2) );
			stones[i++] = stone_create( POS_X(45), POS_Y(3) );
			
			stones[i++] = stone_create( POS_X(50), POS_Y(1) );
			stones[i++] = stone_create( POS_X(50), POS_Y(2) );
			stones[i++] = stone_create( POS_X(50), POS_Y(3) );
			stones[i++] = stone_create( POS_X(54), POS_Y(1) );
			stones[i++] = stone_create( POS_X(55), POS_Y(2) );
			stones[i++] = stone_create( POS_X(54), POS_Y(3) ); // 38
			
			stones[i++] = stone_create( POS_X(59), POS_Y(0) );
			stones[i++] = stone_create( POS_X(59), POS_Y(3) ); // 40
			
			stones[i++] = stone_create( POS_X(64), POS_Y(0) );
			stones[i++] = stone_create( POS_X(64), POS_Y(1) );
			stones[i++] = stone_create( POS_X(64), POS_Y(2) );		
			stones[i++] = stone_create( POS_X(67), POS_Y(0) );
			stones[i++] = stone_create( POS_X(68), POS_Y(1) );
			stones[i++] = stone_create( POS_X(67), POS_Y(2) );
			
			stones[i++] = stone_create( POS_X(72), POS_Y(0) );
			stones[i++] = stone_create( POS_X(72), POS_Y(2) );
			stones[i++] = stone_create( POS_X(72), POS_Y(3) );
			stones[i++] = stone_create( POS_X(76), POS_Y(0) );
			stones[i++] = stone_create( POS_X(75), POS_Y(2) );
			stones[i++] = stone_create( POS_X(75), POS_Y(3) );
			
			stones[i++] = stone_create( POS_X(80), POS_Y(1) );
			stones[i++] = stone_create( POS_X(80), POS_Y(2) );
			stones[i++] = stone_create( POS_X(80), POS_Y(3) );
			stones[i++] = stone_create( POS_X(84), POS_Y(1) );
			stones[i++] = stone_create( POS_X(85), POS_Y(2) );
			stones[i++] = stone_create( POS_X(84), POS_Y(3) ); // 58
			
			stones[i++] = stone_create( POS_X(89), POS_Y(0) );
			stones[i++] = stone_create( POS_X(89), POS_Y(3) ); // 60		
		break;
		case 2:
			srand(time_getTicks()+5645673);
			x = 9;
			x_max = 9;
			
			while(i <= 50) {			
				//rand1 = rand()%(5-3) + 3; // [3;4] make 3 or 4 stones on a line			
				rand1 = rand()%(5-1) + 1;
				
				if(rand1 <= 3) {
					rand2 = rand()%4; // [0;3] POS_Y != this
					for(j = 0; j < 4; j++) {
						if(j != rand2) {
							x_tmp = x + rand()%2; // x += [0;1]
							stones[i++] = stone_create( (x_tmp*8 + 50) , (24+(j*10)) );
							if(x_tmp > x_max) x_max = x_tmp;
						}
					}
				} else if(rand1 == 4) {
					for(j = 0; j < 4; j++) {
						x_tmp = x + rand()%2; // x += [0;1]
						stones[i++] = stone_create( (x_tmp*8 + 50) , (24+(j*10)) );
						if(x_tmp > x_max) x_max = x_tmp;
					}
				}
				x = x_max = x_max + 7;
			}
			while(i < 60) {
				x_tmp = x + rand()%2; // x += [0;1]
				y_tmp = rand()%4;
				stones[i++] = stone_create( (x_tmp*8 + 50), (24+(y_tmp*10)) );
				if(x_tmp > x_max) x_max = x_tmp;
				x = x_max = x_max + 2;
			}		
		break;
	}
}
void map_free(Stone * stones[60]) {
	int i;
	
	for(i = 0; i < 60; i++) {
		if(stones[i] != NULL) {
			free(stones[i]);
			stones[i] = NULL;
		}
	}
}

Bird * bird_create() {
	Bird * bird = (Bird *) malloc(sizeof(Bird));

	bird_reset(bird);
	
	return bird;
}
void bird_reset(Bird * bird) {
	bird->x = -5;
	bird->y = 5;
	bird->dx = 2;
	bird->tick_last = 0;
	bird->position = 1;
}
void bird_draw(Bird * bird) {
	if(bird->position == 1) {
		ML_pixel(bird->x-1, bird->y, ML_BLACK);
		ML_pixel(bird->x, bird->y, ML_BLACK);
		ML_pixel(bird->x+1, bird->y+1, ML_BLACK);
		ML_pixel(bird->x+2, bird->y, ML_BLACK);
		ML_pixel(bird->x+3, bird->y, ML_BLACK);
	} else {
		ML_pixel(bird->x-1, bird->y+1, ML_BLACK);
		ML_pixel(bird->x, bird->y+1, ML_BLACK);
		ML_pixel(bird->x+1, bird->y, ML_BLACK);
		ML_pixel(bird->x+2, bird->y+1, ML_BLACK);
		ML_pixel(bird->x+3, bird->y+1, ML_BLACK);
	}
}
void bird_update(Bird * bird) {
	if(time_getTicks() - bird->tick_last > 10) {
		bird->x += bird->dx;
		
		if(bird->x < -5 || bird->x > 132) { // new bird
			bird->dx = rand()%5 - 2;
			if(bird->dx == 0) bird->dx = 1;
			
			if(bird->dx < 0)
				bird->x = 132;
			else
				bird->x = -5;
				
			bird->y = rand()%(10-5) + 5;
		}
		
		if(bird->position == 1)
			bird->position = 2;
		else
			bird->position = 1;
		
		bird->tick_last = time_getTicks();
	}
}

Cloud * cloud_create(unsigned char ** bmp) {
	Cloud * cloud = (Cloud *) malloc(sizeof(Cloud));
	
	cloud->bmp = bmp;
	
	cloud_reset(cloud);
	
	return cloud;
}
void cloud_reset(Cloud * cloud) {
	cloud->x = 1;
	cloud->y = 4;
	cloud->dx = 1;
	cloud->tick_last = 0;
}
void cloud_draw(Cloud * cloud) {	
	ML_bmp_16_or_cl(cloud->bmp[0], cloud->x, cloud->y);	
	ML_bmp_16_and_cl(cloud->bmp[1], cloud->x, cloud->y);
}
void cloud_update(Cloud * cloud) {
	if(time_getTicks() - cloud->tick_last > 130) {
		cloud->x += cloud->dx;
		
		if(cloud->x < -19 || cloud->x > 140) { // new bird
			cloud->dx = rand()%3 - 2;
			if(cloud->dx == 0) cloud->dx = 1;
			
			if(cloud->dx < 0)
				cloud->x = 140;
			else
				cloud->x = -19;
				
			cloud->y = rand()%(10-5) + 5;
		}
		
		cloud->tick_last = time_getTicks();
	}
}